package cn.iocoder.yudao.module.attendance.controller.admin.out;

import cn.iocoder.yudao.module.infra.api.file.FileApi;
import cn.iocoder.yudao.module.infra.api.file.dto.FileDTO;
import cn.iocoder.yudao.module.system.api.dept.DeptApi;
import cn.iocoder.yudao.module.system.api.dept.dto.DeptRespDTO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Operation;

import java.util.*;
import java.io.IOException;
import java.util.stream.Collectors;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.*;

import cn.iocoder.yudao.module.attendance.controller.admin.out.vo.*;
import cn.iocoder.yudao.module.attendance.dal.dataobject.out.AttendanceOutDO;
import cn.iocoder.yudao.module.attendance.service.out.AttendanceOutService;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

@Tag(name = "管理后台 - 公务外出信息")
@RestController
@RequestMapping("/personnel/attendance/out")
@Validated
public class AttendanceOutController {

    @Resource
    private AttendanceOutService outService;
    @Resource
    private DeptApi deptApi;
    @Resource
    private FileApi fileApi;

    @PostMapping("/create")
    @Operation(summary = "创建公务外出信息")
    @PreAuthorize("@ss.hasPermission('attendance:out:create')")
    public CommonResult<Long> createOut(@Valid @RequestBody AttendanceOutSaveReqVO createReqVO) {
        return success(outService.createOut(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新公务外出信息")
    @PreAuthorize("@ss.hasPermission('attendance:out:update')")
    public CommonResult<Boolean> updateOut(@Valid @RequestBody AttendanceOutSaveReqVO updateReqVO) {
        outService.updateOut(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除公务外出信息")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('attendance:out:delete')")
    public CommonResult<Boolean> deleteOut(@RequestParam("id") Long id) {
        outService.deleteOut(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得公务外出信息")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('attendance:out:query')")
    public CommonResult<AttendanceOutRespVO> getOut(@RequestParam("id") Long id) {
        AttendanceOutDO out = outService.getOut(id);
        AttendanceOutRespVO respVO = BeanUtils.toBean(out, AttendanceOutRespVO.class);
        // 部门
        if (respVO.getDeptId() != null) {
            DeptRespDTO dept = deptApi.getDept(respVO.getDeptId());
            respVO.setDeptName(dept.getName());
            if (dept != null && StringUtils.isNotBlank(dept.getName())) {
                respVO.setDeptName(dept.getName());
            }
        }
        // 附件列表
        List<FileDTO> fileList = fileApi.getFileDTOListByBiz(respVO.getOutId());
        respVO.setFileList(fileList);
        List<Long> fileIdList = fileList.stream().map(FileDTO::getId).collect(Collectors.toList());
        respVO.setFileIdList(fileIdList);
        return success(respVO);
    }

    @GetMapping("/page")
    @Operation(summary = "获得公务外出信息分页")
    @PreAuthorize("@ss.hasPermission('attendance:out:query')")
    public CommonResult<PageResult<AttendanceOutRespVO>> getOutPage(@Valid AttendanceOutPageReqVO pageReqVO) {
        PageResult<AttendanceOutDO> pageResult = outService.getOutPage(pageReqVO);
        PageResult<AttendanceOutRespVO> result = BeanUtils.toBean(pageResult, AttendanceOutRespVO.class);
        if (result != null && result.getList() != null && result.getList().size() > 0) {
            result.getList().forEach(respVO -> {
                // 部门
                if (respVO.getDeptId() != null) {
                    DeptRespDTO dept = deptApi.getDept(respVO.getDeptId());
                    if (dept != null && StringUtils.isNotBlank(dept.getName())) {
                        respVO.setDeptName(dept.getName());
                    }
                }
            });
        }
        return success(result);
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出公务外出信息 Excel")
    @PreAuthorize("@ss.hasPermission('attendance:out:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportOutExcel(@Valid AttendanceOutPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        PageResult<AttendanceOutDO> pageResult = outService.getOutPage(pageReqVO);
        PageResult<AttendanceOutRespVO> result = BeanUtils.toBean(pageResult, AttendanceOutRespVO.class);
        if (result != null && result.getList() != null && result.getList().size() > 0) {
            result.getList().forEach(respVO -> {
                // 部门
                if (respVO.getDeptId() != null) {
                    DeptRespDTO dept = deptApi.getDept(respVO.getDeptId());
                    if (dept != null && StringUtils.isNotBlank(dept.getName())) {
                        respVO.setDeptName(dept.getName());
                    }
                }
            });
        }
        // 导出 Excel
        ExcelUtils.write(response, "公务外出信息.xls", "数据", AttendanceOutRespVO.class,
                result.getList());
    }

}