package cn.iocoder.yudao.module.attendance.controller.admin.workdaysetting;

import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Operation;

import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.*;

import cn.iocoder.yudao.module.attendance.controller.admin.workdaysetting.vo.*;
import cn.iocoder.yudao.module.attendance.dal.dataobject.workdaysetting.AttendanceWorkdaySettingDO;
import cn.iocoder.yudao.module.attendance.service.workdaysetting.AttendanceWorkdaySettingService;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

@Tag(name = "管理后台 - 考勤工作日设置")
@RestController
@RequestMapping("/personnel/attendance/workday-setting")
@Validated
public class AttendanceWorkdaySettingController {

    @Resource
    private AttendanceWorkdaySettingService workdaySettingService;

    @PostMapping("/create")
    @Operation(summary = "创建考勤工作日设置")
    @PreAuthorize("@ss.hasPermission('attendance:workday-setting:create')")
    public CommonResult<Long> createWorkdaySetting(@Valid @RequestBody AttendanceWorkdaySettingSaveReqVO createReqVO) {
        return success(workdaySettingService.createWorkdaySetting(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新考勤工作日设置")
    @PreAuthorize("@ss.hasPermission('attendance:workday-setting:update')")
    public CommonResult<Boolean> updateWorkdaySetting(@Valid @RequestBody AttendanceWorkdaySettingSaveReqVO updateReqVO) {
        workdaySettingService.updateWorkdaySetting(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除考勤工作日设置")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('attendance:workday-setting:delete')")
    public CommonResult<Boolean> deleteWorkdaySetting(@RequestParam("id") Long id) {
        workdaySettingService.deleteWorkdaySetting(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得考勤工作日设置")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('attendance:workday-setting:query')")
    public CommonResult<AttendanceWorkdaySettingRespVO> getWorkdaySetting(@RequestParam("id") Long id) {
        AttendanceWorkdaySettingDO workdaySetting = workdaySettingService.getWorkdaySetting(id);
        return success(BeanUtils.toBean(workdaySetting, AttendanceWorkdaySettingRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得考勤工作日设置分页")
    @PreAuthorize("@ss.hasPermission('attendance:workday-setting:query')")
    public CommonResult<PageResult<AttendanceWorkdaySettingRespVO>> getWorkdaySettingPage(@Valid AttendanceWorkdaySettingPageReqVO pageReqVO) {
        PageResult<AttendanceWorkdaySettingDO> pageResult = workdaySettingService.getWorkdaySettingPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, AttendanceWorkdaySettingRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出考勤工作日设置 Excel")
    @PreAuthorize("@ss.hasPermission('attendance:workday-setting:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportWorkdaySettingExcel(@Valid AttendanceWorkdaySettingPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<AttendanceWorkdaySettingDO> list = workdaySettingService.getWorkdaySettingPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "考勤工作日设置.xls", "数据", AttendanceWorkdaySettingRespVO.class,
                        BeanUtils.toBean(list, AttendanceWorkdaySettingRespVO.class));
    }

    @PostMapping("/batch-insert")
    @Operation(summary = "批量设置考勤工作日")
    @PreAuthorize("@ss.hasPermission('attendance:workday-setting:insert')")
    public CommonResult<Boolean> batchInsert(@RequestBody AttendanceWorkdaySettingDatesReqVO reqVO) {
        return success(workdaySettingService.batchInsert(reqVO.getDates()));
    }

    @GetMapping("/get-workdays")
    @Operation(summary = "获得考勤工作日数据")
    @PreAuthorize("@ss.hasPermission('attendance:workday-setting:query')")
    public CommonResult<List<AttendanceWorkdaySettingRespVO>> getWorkdays(@RequestParam("workMonth") String workMonth) {
        List<AttendanceWorkdaySettingDO> list = workdaySettingService.getWorkdays(workMonth);
        return success(BeanUtils.toBean(list, AttendanceWorkdaySettingRespVO.class));
    }

}