package cn.iocoder.yudao.module.relations.controller.admin.conversion;

import cn.iocoder.yudao.module.system.api.dept.DeptApi;
import cn.iocoder.yudao.module.system.api.dept.dto.DeptRespDTO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Operation;

import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.*;

import cn.iocoder.yudao.module.relations.controller.admin.conversion.vo.*;
import cn.iocoder.yudao.module.relations.dal.dataobject.conversion.RelationsConversionDO;
import cn.iocoder.yudao.module.relations.service.conversion.RelationsConversionService;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

@Tag(name = "管理后台 - 转正信息")
@RestController
@RequestMapping("/personnel/relations/conversion")
@Validated
public class RelationsConversionController {

    @Resource
    private RelationsConversionService conversionService;
    @Resource
    private DeptApi deptApi;

    @PostMapping("/create")
    @Operation(summary = "创建转正信息")
    @PreAuthorize("@ss.hasPermission('relations:conversion:create')")
    public CommonResult<Long> createConversion(@Valid @RequestBody RelationsConversionSaveReqVO createReqVO) {
        return success(conversionService.createConversion(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新转正信息")
    @PreAuthorize("@ss.hasPermission('relations:conversion:update')")
    public CommonResult<Boolean> updateConversion(@Valid @RequestBody RelationsConversionSaveReqVO updateReqVO) {
        conversionService.updateConversion(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除转正信息")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('relations:conversion:delete')")
    public CommonResult<Boolean> deleteConversion(@RequestParam("id") Long id) {
        conversionService.deleteConversion(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得转正信息")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('relations:conversion:query')")
    public CommonResult<RelationsConversionRespVO> getConversion(@RequestParam("id") Long id) {
        return success(conversionService.getById(id));
    }

    @GetMapping("/page")
    @Operation(summary = "获得转正信息分页")
    @PreAuthorize("@ss.hasPermission('relations:conversion:query')")
    public CommonResult<PageResult<RelationsConversionRespVO>> getConversionPage(@Valid RelationsConversionPageReqVO pageReqVO) {
        PageResult<RelationsConversionDO> pageResult = conversionService.getConversionPage(pageReqVO);
        PageResult<RelationsConversionRespVO> result = BeanUtils.toBean(pageResult, RelationsConversionRespVO.class);
        if (result != null && result.getList() != null && result.getList().size() > 0) {
            result.getList().forEach(respVO -> {
                // 部门
                if (respVO.getDeptId() != null) {
                    DeptRespDTO dept = deptApi.getDept(respVO.getDeptId());
                    if (dept != null && StringUtils.isNotBlank(dept.getName())) {
                        respVO.setDeptName(dept.getName());
                    }
                }
            });
        }
        return success(result);
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出转正信息 Excel")
    @PreAuthorize("@ss.hasPermission('relations:conversion:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportConversionExcel(@Valid RelationsConversionPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        PageResult<RelationsConversionDO> pageResult = conversionService.getConversionPage(pageReqVO);
        PageResult<RelationsConversionRespVO> result = BeanUtils.toBean(pageResult, RelationsConversionRespVO.class);
        if (result != null && result.getList() != null && result.getList().size() > 0) {
            result.getList().forEach(respVO -> {
                // 部门
                if (respVO.getDeptId() != null) {
                    DeptRespDTO dept = deptApi.getDept(respVO.getDeptId());
                    if (dept != null && StringUtils.isNotBlank(dept.getName())) {
                        respVO.setDeptName(dept.getName());
                    }
                }
            });
        }
        // 导出 Excel
        ExcelUtils.write(response, "转正信息.xls", "数据", RelationsConversionRespVO.class,
                result.getList());
    }

}