package cn.iocoder.yudao.module.system.controller.admin.auth;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.enums.CommonStatusEnum;
import cn.iocoder.yudao.framework.common.enums.UserTypeEnum;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.security.config.SecurityProperties;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.employee.api.EmployeeApi;
import cn.iocoder.yudao.module.employee.api.dto.EmployeeRespDTO;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserRespDTO;
import cn.iocoder.yudao.module.system.controller.admin.auth.vo.*;
import cn.iocoder.yudao.module.system.convert.auth.AuthConvert;
import cn.iocoder.yudao.module.system.dal.dataobject.permission.MenuDO;
import cn.iocoder.yudao.module.system.dal.dataobject.permission.RoleDO;
import cn.iocoder.yudao.module.system.dal.dataobject.tenant.TenantDO;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;
import cn.iocoder.yudao.module.system.enums.logger.LoginLogTypeEnum;
import cn.iocoder.yudao.module.system.service.auth.AdminAuthService;
import cn.iocoder.yudao.module.system.service.permission.MenuService;
import cn.iocoder.yudao.module.system.service.permission.PermissionService;
import cn.iocoder.yudao.module.system.service.permission.RoleService;
import cn.iocoder.yudao.module.system.service.social.SocialClientService;
import cn.iocoder.yudao.module.system.service.tenant.TenantService;
import cn.iocoder.yudao.module.system.service.user.AdminUserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.annotation.security.PermitAll;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.Set;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.common.util.collection.CollectionUtils.convertSet;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;

@Tag(name = "管理后台 - 认证")
@RestController
@RequestMapping("/system/auth")
@Validated
@Slf4j
public class AuthController {

    @Resource
    private AdminAuthService authService;
    @Resource
    private AdminUserService userService;
    @Resource
    private RoleService roleService;
    @Resource
    private MenuService menuService;
    @Resource
    private PermissionService permissionService;
    @Resource
    private SocialClientService socialClientService;

    @Resource
    private SecurityProperties securityProperties;
    @Resource
    private TenantService tenantService;

    @Resource
    private EmployeeApi employeeApi;

    @PostMapping("/login")
    @PermitAll
    @Operation(summary = "使用账号密码登录")
    public CommonResult<AuthLoginRespVO> login(@RequestBody @Valid AuthLoginReqVO reqVO) {
        return success(authService.login(reqVO));
    }

    @PostMapping("/logout")
    @PermitAll
    @Operation(summary = "登出系统")
    public CommonResult<Boolean> logout(HttpServletRequest request) {
        String token = SecurityFrameworkUtils.obtainAuthorization(request,
                securityProperties.getTokenHeader(), securityProperties.getTokenParameter());
        if (StrUtil.isNotBlank(token)) {
            authService.logout(token, LoginLogTypeEnum.LOGOUT_SELF.getType());
        }
        return success(true);
    }

    @PostMapping("/refresh-token")
    @PermitAll
    @Operation(summary = "刷新令牌")
    @Parameter(name = "refreshToken", description = "刷新令牌", required = true)
    public CommonResult<AuthLoginRespVO> refreshToken(@RequestParam("refreshToken") String refreshToken) {
        return success(authService.refreshToken(refreshToken));
    }

    @GetMapping("/get-permission-info")
    @Operation(summary = "获取登录用户对应员工的权限信息")
    public CommonResult<AuthPermissionInfoRespVO> getPermissionInfo() {
        // 1.1 获得用户信息
        AdminUserDO user = userService.getUser(getLoginUserId());
        if (user == null) {
            return success(null);
        }
        AdminUserRespDTO dto = BeanUtils.toBean(user, AdminUserRespDTO.class);
        // 根据用户ID查询员工信息
        EmployeeRespDTO employee = employeeApi.getEmployeeByUserId(user.getId());
        if (employee != null) {
            // 设置员工ID
            dto.setId(employee.getId());
            // 设置员工昵称
            dto.setNickname(employee.getName());
            // 设置员工头像
            dto.setAvatar(employee.getAvatar());
            // 设置员工部门ID
            dto.setDeptId(employee.getDeptId());
            // 设置员工部门名称
            dto.setDeptName(employee.getDeptName());
            // 设置员工职位ID
            dto.setPostId(employee.getPostId());
            // 设置员工职位名称
            dto.setPosition(employee.getPosition());
            // 设置员工手机号
            dto.setMobile(employee.getPhone());
        }

        TenantDO tenantDO = tenantService.getTenant(dto.getTenantId()); // 获得租户信息，用于判断是否需要切换租户
        if (tenantDO != null && tenantDO.getName() != null) {
            dto.setTenantName(tenantDO.getName());
            dto.setCorpId(tenantDO.getCorpId());
        }

        // 1.2 获得角色列表
        Set<Long> roleIds = permissionService.getUserRoleIdListByUserId(getLoginUserId());
        if (CollUtil.isEmpty(roleIds)) {
            return success(AuthConvert.INSTANCE.convert(dto, Collections.emptyList(), Collections.emptyList()));
        }
        List<RoleDO> roles = roleService.getRoleList(roleIds);
        roles.removeIf(role -> !CommonStatusEnum.ENABLE.getStatus().equals(role.getStatus())); // 移除禁用的角色

        // 1.3 获得菜单列表
        Set<Long> menuIds = permissionService.getRoleMenuListByRoleId(convertSet(roles, RoleDO::getId));
        List<MenuDO> menuList = menuService.getMenuList(menuIds);
        menuList.removeIf(menu -> !CommonStatusEnum.ENABLE.getStatus().equals(menu.getStatus())); // 移除禁用的菜单
        Optional<MenuDO> manageMenu = menuList.stream()
                .filter(menu -> "企业管理".equals(menu.getName())) // 查找name为“企业管理”的MenuDO
                .findFirst(); // 返回第一个找到的MenuDO（如果有的话）
        dto.setManageMenuId(manageMenu.map(MenuDO::getId).orElse(null));
        // 2. 拼接结果返回
        return success(AuthConvert.INSTANCE.convert(dto, roles, menuList));
    }

    // ========== 短信登录相关 ==========

    @PostMapping("/sms-login")
    @PermitAll
    @Operation(summary = "使用短信验证码登录")
    public CommonResult<AuthLoginRespVO> smsLogin(@RequestBody @Valid AuthSmsLoginReqVO reqVO) {
        return success(authService.smsLogin(reqVO));
    }

    @PostMapping("/send-sms-code")
    @PermitAll
    @Operation(summary = "发送手机验证码")
    public CommonResult<Boolean> sendLoginSmsCode(@RequestBody @Valid AuthSmsSendReqVO reqVO) {
        authService.sendSmsCode(reqVO);
        return success(true);
    }

    @PostMapping("/sms-register")
    @PermitAll
    @Operation(summary = "使用验证码注册并登录")
    public CommonResult<AuthLoginRespVO> smsRegister(@RequestBody @Valid AuthSmsLoginReqVO reqVO) {
        return success(authService.smsRegister(reqVO));
    }

    @PostMapping("/sms-reset-password")
    @PermitAll
    @Operation(summary = "使用验证码重置密码")
    public CommonResult<AuthLoginRespVO> smsResetPassword(@RequestBody @Valid AuthSmsLoginReqVO reqVO) {
        return success(authService.smsResetPassword(reqVO));
    }

    // ========== 社交登录相关 ==========

    @GetMapping("/social-auth-redirect")
    @PermitAll
    @Operation(summary = "社交授权的跳转")
    @Parameters({
            @Parameter(name = "type", description = "社交类型", required = true),
            @Parameter(name = "redirectUri", description = "回调路径")
    })
    public CommonResult<String> socialLogin(@RequestParam("type") Integer type,
                                            @RequestParam("redirectUri") String redirectUri) {
        return success(socialClientService.getAuthorizeUrl(
                type, UserTypeEnum.ADMIN.getValue(), redirectUri));
    }

    @PostMapping("/social-login")
    @PermitAll
    @Operation(summary = "社交快捷登录，使用 code 授权码", description = "适合未登录的用户，但是社交账号已绑定用户")
    public CommonResult<AuthLoginRespVO> socialQuickLogin(@RequestBody @Valid AuthSocialLoginReqVO reqVO) {
        return success(authService.socialLogin(reqVO));
    }

    @GetMapping("/getCurrentTenantIdByUsername")
    @PermitAll
    @Operation(summary = "通过用户账号获取租户列表")
    @Parameter(name = "username", description = "用户账号", required = true, example = "1024")
    public CommonResult<Long> getCurrentTenantIdByUsername(@RequestParam("username") String username) {
        return success(authService.getCurrentTenantIdByUsername(username));
    }

    private String getEnterpriseManagementMenuId(List<MenuDO> menuList) {
        // 使用Java 8的Stream API来找到第一个匹配的MenuDO
        Optional<MenuDO> foundMenu = menuList.stream()
                .filter(menu -> "企业管理".equals(menu.getName())) // 查找name为“企业管理”的MenuDO
                .findFirst(); // 返回第一个找到的MenuDO（如果有的话）

        // 如果找到了，就返回它的id，否则返回null
        return String.valueOf(foundMenu.map(MenuDO::getId).orElse(null));
    }

}
